@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import Literal, Sequence, Union
+from typing import Literal, Sequence, Union, Optional
 from dataclasses import dataclass, field
 from .tokens import Token, Tokens
 
@@ -132,6 +132,9 @@ class CDDLNode:
             return token.serialize()
         return marker.serializeToken(token, self)
 
+    def __repr__(self, indent: int = 0) -> str:
+        return "  " * indent + self.__class__.__name__
+
 
 class WrappedNode(CDDLNode):
     """
@@ -153,6 +156,14 @@ class WrappedNode(CDDLNode):
         """
         raise NotImplementedError("_serialize method must be implemented in subclass")
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        if self.openToken is not None:
+            res.append(self.openToken.__repr__(indent + 1))
+        if self.closeToken is not None:
+            res.append(self.closeToken.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 class TokenNode(WrappedNode):
     """
@@ -208,6 +219,16 @@ class TokenNode(WrappedNode):
         self.comments = token.comments
         self.whitespace = token.whitespace
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for comment in self.comments:
+            res.append(comment.__repr__(indent + 1))
+        if self.whitespace != "":
+            res.append("  " * (indent + 1) + f"whitespaces: {len(self.whitespace)}")
+        if self.separator is not None:
+            res.append(self.separator.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class CDDLTree(TokenNode):
@@ -217,7 +238,7 @@ class CDDLTree(TokenNode):
 
     rules: list[Rule]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -226,6 +247,12 @@ class CDDLTree(TokenNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.rules])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for rule in self.rules:
+            res.append(rule.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Rule(CDDLNode):
@@ -246,7 +273,7 @@ class Rule(CDDLNode):
     assign: Token
     type: Type | GroupEntry
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -258,6 +285,16 @@ class Rule(CDDLNode):
         output += self.type.serialize(marker)
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        return "\n".join(
+            [
+                super().__repr__(indent),
+                self.name.__repr__(indent + 1),
+                self.assign.__repr__(indent + 1),
+                self.type.__repr__(indent + 1),
+            ]
+        )
+
 
 @dataclass
 class GroupEntry(TokenNode):
@@ -269,7 +306,7 @@ class GroupEntry(TokenNode):
     key: Memberkey | None
     type: Type
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -308,6 +345,15 @@ class GroupEntry(TokenNode):
             )
         )
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        if self.occurrence is not None:
+            res.append(self.occurrence.__repr__(indent + 1))
+        if self.key is not None:
+            res.append(self.key.__repr__(indent + 1))
+        res.append(self.type.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Group(TokenNode):
@@ -317,7 +363,7 @@ class Group(TokenNode):
 
     groupChoices: list[GroupChoice]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -326,15 +372,19 @@ class Group(TokenNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.groupChoices])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.groupChoices:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
-@dataclass
 class Map(Group):
     """
     A map, meaning a list of group choices wrapped in curly braces
     """
 
 
-@dataclass
 class Array(Group):
     """
     An array
@@ -352,7 +402,7 @@ class GroupChoice(TokenNode):
 
     groupEntries: list[GroupEntry]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -361,6 +411,12 @@ class GroupChoice(TokenNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.groupEntries])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.groupEntries:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Tag(TokenNode):
@@ -376,7 +432,7 @@ class Tag(TokenNode):
     numericPart: Token | None = None
     typePart: Type | None = None
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -388,6 +444,14 @@ class Tag(TokenNode):
         output += self.typePart.serialize(marker) if self.typePart is not None else ""
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent) + " (#)"]
+        if self.numericPart is not None:
+            res.append(self.numericPart.__repr__(indent + 1))
+        if self.typePart is not None:
+            res.append(self.typePart.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Occurrence(TokenNode):
@@ -398,12 +462,18 @@ class Occurrence(TokenNode):
     # different combinations right
     tokens: list[Token] = field(default_factory=list)
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([self._serializeToken(item, marker) for item in self.tokens])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.tokens:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Value(TokenNode):
@@ -414,7 +484,7 @@ class Value(TokenNode):
     value: str
     type: ValueType
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def _serialize(self, marker: Marker | None = None) -> str:
@@ -436,6 +506,9 @@ class Value(TokenNode):
             return prefix + self.value + suffix
         return marker.serializeValue(prefix, self.value, suffix, self)
 
+    def __repr__(self, indent: int = 0) -> str:
+        return "  " * indent + f"{self.__class__.__name__} ({self.type}): {self.value}"
+
 
 @dataclass
 class Typename(TokenNode):
@@ -447,7 +520,7 @@ class Typename(TokenNode):
     unwrapped: Token | None
     parameters: GenericParameters | GenericArguments | None = None
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -466,6 +539,14 @@ class Typename(TokenNode):
             output += self.parameters.serialize(marker)
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent) + ": " + self.name]
+        if self.unwrapped is not None:
+            res.append(self.unwrapped.__repr__(indent + 1))
+        if self.parameters is not None:
+            res.append(self.parameters.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class ChoiceFrom(TokenNode):
@@ -475,7 +556,7 @@ class ChoiceFrom(TokenNode):
 
     target: Group | Typename
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -486,12 +567,19 @@ class ChoiceFrom(TokenNode):
         output += self.target.serialize(marker)
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [
+            super().__repr__(indent) + " (&)",
+            self.target.__repr__(indent + 1),
+        ]
+        return "\n".join(res)
+
 
 # A type2 production is one of a few possibilities
 # (Note the need to use a forward reference to resolve the circular dependency
 # between Type and Type2, see:
 # https://docs.python.org/3/library/stdtypes.html#types-union)
-Type2 = Union["Value | Typename | Type | Group | Map | Array | ChoiceFrom | Tag"]
+Type2 = Union[Value, Typename, "Type", Group, Map, Array, ChoiceFrom, Tag]
 
 
 @dataclass
@@ -510,7 +598,7 @@ class Range(TokenNode):
     # for now to store spaces and comments
     rangeop: Token
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -522,6 +610,15 @@ class Range(TokenNode):
         output += self.max.serialize(marker)
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [
+            super().__repr__(indent),
+            self.min.__repr__(indent + 1),
+            self.rangeop.__repr__(indent + 1),
+            self.max.__repr__(indent + 1),
+        ]
+        return "\n".join(res)
+
 
 @dataclass
 class Operator(TokenNode):
@@ -535,7 +632,7 @@ class Operator(TokenNode):
     name: Token
     controller: Type2
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -547,9 +644,18 @@ class Operator(TokenNode):
         output += self.controller.serialize(marker)
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [
+            super().__repr__(indent),
+            self.type.__repr__(indent + 1),
+            self.name.__repr__(indent + 1),
+            self.controller.__repr__(indent + 1),
+        ]
+        return "\n".join(res)
+
 
 # A Type1 production is either a Type2, a Range or an Operator
-Type1 = Type2 | Range | Operator
+Type1 = Union[Type2, Range, Operator]
 
 
 @dataclass
@@ -562,7 +668,7 @@ class Memberkey(CDDLNode):
     # different combinations of cut tokens right (with spaces and comments)
     tokens: list[Token] = field(default_factory=list)
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -575,6 +681,12 @@ class Memberkey(CDDLNode):
         )
         return output
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.tokens:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class Type(TokenNode):
@@ -587,7 +699,7 @@ class Type(TokenNode):
 
     types: list[Type1]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -596,6 +708,12 @@ class Type(TokenNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.types])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.types:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class GenericParameters(WrappedNode):
@@ -605,7 +723,7 @@ class GenericParameters(WrappedNode):
 
     parameters: list[Typename]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -614,6 +732,12 @@ class GenericParameters(WrappedNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.parameters])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.parameters:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
+
 
 @dataclass
 class GenericArguments(WrappedNode):
@@ -623,7 +747,7 @@ class GenericArguments(WrappedNode):
 
     parameters: list[Type1]
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         super().__init__()
 
     def getChildren(self) -> Sequence[CDDLNode]:
@@ -632,8 +756,14 @@ class GenericArguments(WrappedNode):
     def _serialize(self, marker: Marker | None = None) -> str:
         return "".join([item.serialize(marker) for item in self.parameters])
 
+    def __repr__(self, indent: int = 0) -> str:
+        res: list[str] = [super().__repr__(indent)]
+        for item in self.parameters:
+            res.append(item.__repr__(indent + 1))
+        return "\n".join(res)
 
-Markup = tuple[str | None, str | None]
+
+Markup = tuple[Optional[str], Optional[str]]
 
 
 class Marker:
