@@ -38,10 +38,8 @@ class Lexer:
         sourceLines = self.input.split("\n")
         sourceLineLength = [len(line) for line in sourceLines]
         i = 0
-
         for line, lineLength in enumerate(sourceLineLength):
             i += lineLength + 1
-            lineBegin = i - lineLength
             if i > position:
                 lineBegin = i - lineLength
                 return Location(line, position - lineBegin + 1)
@@ -68,126 +66,120 @@ class Lexer:
         token = Token(Tokens.ILLEGAL, "")
         literal = chr(self.ch)
         tokenRead = False
-        match literal:
-            case "=":
-                if self._peekAtNextChar() == ">":
-                    self.readChar()
-                    token = Token(Tokens.ARROWMAP, "", comments, whitespace)
-                else:
-                    token = Token(Tokens.ASSIGN, "", comments, whitespace)
-            case "(":
-                token = Token(Tokens.LPAREN, "", comments, whitespace)
-            case ")":
-                token = Token(Tokens.RPAREN, "", comments, whitespace)
-            case "{":
-                token = Token(Tokens.LBRACE, "", comments, whitespace)
-            case "}":
-                token = Token(Tokens.RBRACE, "", comments, whitespace)
-            case "[":
-                token = Token(Tokens.LBRACK, "", comments, whitespace)
-            case "]":
-                token = Token(Tokens.RBRACK, "", comments, whitespace)
-            case "<":
-                token = Token(Tokens.LT, "", comments, whitespace)
-            case ">":
-                token = Token(Tokens.GT, "", comments, whitespace)
-            case "+":
-                token = Token(Tokens.PLUS, "", comments, whitespace)
-            case ",":
-                token = Token(Tokens.COMMA, "", comments, whitespace)
-            case ".":
+        if literal == "=":
+            if self._peekAtNextChar() == ">":
+                self.readChar()
+                token = Token(Tokens.ARROWMAP, "", comments, whitespace)
+            else:
+                token = Token(Tokens.ASSIGN, "", comments, whitespace)
+        elif literal == "(":
+            token = Token(Tokens.LPAREN, "", comments, whitespace)
+        elif literal == ")":
+            token = Token(Tokens.RPAREN, "", comments, whitespace)
+        elif literal == "{":
+            token = Token(Tokens.LBRACE, "", comments, whitespace)
+        elif literal == "}":
+            token = Token(Tokens.RBRACE, "", comments, whitespace)
+        elif literal == "[":
+            token = Token(Tokens.LBRACK, "", comments, whitespace)
+        elif literal == "]":
+            token = Token(Tokens.RBRACK, "", comments, whitespace)
+        elif literal == "<":
+            token = Token(Tokens.LT, "", comments, whitespace)
+        elif literal == ">":
+            token = Token(Tokens.GT, "", comments, whitespace)
+        elif literal == "+":
+            token = Token(Tokens.PLUS, "", comments, whitespace)
+        elif literal == ",":
+            token = Token(Tokens.COMMA, "", comments, whitespace)
+        elif literal == ".":
+            if self._peekAtNextChar() == ".":
+                self.readChar()
+                token = Token(Tokens.INCLRANGE, "", comments, whitespace)
                 if self._peekAtNextChar() == ".":
                     self.readChar()
-                    token = Token(Tokens.INCLRANGE, "", comments, whitespace)
-                    if self._peekAtNextChar() == ".":
-                        self.readChar()
-                        token = Token(Tokens.EXCLRANGE, "", comments, whitespace)
-                elif isExtendedAlpha(self._peekAtNextChar()):
-                    self.readChar()
-                    token = Token(
-                        Tokens.CTLOP, self._readIdentifier(), comments, whitespace
-                    )
-                    tokenRead = True
-            case ":":
-                token = Token(Tokens.COLON, "", comments, whitespace)
-            case "?":
-                token = Token(Tokens.QUEST, "", comments, whitespace)
-            case "/":
-                if self._peekAtNextChar() == "/":
-                    self.readChar()
-                    token = Token(Tokens.GCHOICE, "", comments, whitespace)
-                    if self._peekAtNextChar() == "=":
-                        self.readChar()
-                        token = Token(Tokens.GCHOICEALT, "", comments, whitespace)
-                elif self._peekAtNextChar() == "=":
-                    self.readChar()
-                    token = Token(Tokens.TCHOICEALT, "", comments, whitespace)
-                else:
-                    token = Token(Tokens.TCHOICE, "", comments, whitespace)
-            case "*":
-                token = Token(Tokens.ASTERISK, "", comments, whitespace)
-            case "^":
-                token = Token(Tokens.CARET, "", comments, whitespace)
-            case "#":
-                token = Token(Tokens.HASH, "", comments, whitespace)
-            case "~":
-                token = Token(Tokens.TILDE, "", comments, whitespace)
-            case '"':
-                token = Token(Tokens.STRING, self._readString(), comments, whitespace)
-            case "'":
+                    token = Token(Tokens.EXCLRANGE, "", comments, whitespace)
+            elif isExtendedAlpha(self._peekAtNextChar()):
+                self.readChar()
                 token = Token(
-                    Tokens.BYTES, self._readBytesString(), comments, whitespace
+                    Tokens.CTLOP, self._readIdentifier(), comments, whitespace
                 )
-            case ";":
-                token = Token(Tokens.COMMENT, self._readComment(), comments, whitespace)
                 tokenRead = True
-            case "&":
-                token = Token(Tokens.AMPERSAND, "", comments, whitespace)
-            case _:
-                if self.ch == 0:
-                    token = Token(Tokens.EOF, "", comments, whitespace)
-                elif isExtendedAlpha(literal):
-                    if literal == "b" and self._peekAtNextChar() == "6":
-                        self.readChar()
-                        self.readChar()
-                        if chr(self.ch) == "4" and self._peekAtNextChar() == "'":
-                            self.readChar()
-                            token = Token(
-                                Tokens.BASE64,
-                                self._readBytesString(),
-                                comments,
-                                whitespace,
-                            )
-                        else:
-                            # Looked like a b64 byte string, but that's just a regular
-                            # identifier in the end
-                            token = Token(
-                                Tokens.IDENT,
-                                self._readIdentifier("b6"),
-                                comments,
-                                whitespace,
-                            )
-                            tokenRead = True
-                    elif literal == "h" and self._peekAtNextChar() == "'":
-                        self.readChar()
-                        token = Token(
-                            Tokens.HEX, self._readBytesString(), comments, whitespace
-                        )
-                    else:
-                        token = Token(
-                            Tokens.IDENT, self._readIdentifier(), comments, whitespace
-                        )
-                        tokenRead = True
-                elif literal.isdigit() or literal == "-":
-                    # Numbers start with a digit or a minus
-                    numberOrFloat = self._readNumberOrFloat()
+        elif literal == ":":
+            token = Token(Tokens.COLON, "", comments, whitespace)
+        elif literal == "?":
+            token = Token(Tokens.QUEST, "", comments, whitespace)
+        elif literal == "/":
+            if self._peekAtNextChar() == "/":
+                self.readChar()
+                token = Token(Tokens.GCHOICE, "", comments, whitespace)
+                if self._peekAtNextChar() == "=":
+                    self.readChar()
+                    token = Token(Tokens.GCHOICEALT, "", comments, whitespace)
+            elif self._peekAtNextChar() == "=":
+                self.readChar()
+                token = Token(Tokens.TCHOICEALT, "", comments, whitespace)
+            else:
+                token = Token(Tokens.TCHOICE, "", comments, whitespace)
+        elif literal == "*":
+            token = Token(Tokens.ASTERISK, "", comments, whitespace)
+        elif literal == "^":
+            token = Token(Tokens.CARET, "", comments, whitespace)
+        elif literal == "#":
+            token = Token(Tokens.HASH, "", comments, whitespace)
+        elif literal == "~":
+            token = Token(Tokens.TILDE, "", comments, whitespace)
+        elif literal == '"':
+            token = Token(Tokens.STRING, self._readString(), comments, whitespace)
+        elif literal == "'":
+            token = Token(Tokens.BYTES, self._readBytesString(), comments, whitespace)
+        elif literal == ";":
+            token = Token(Tokens.COMMENT, self._readComment(), comments, whitespace)
+            tokenRead = True
+        elif literal == "&":
+            token = Token(Tokens.AMPERSAND, "", comments, whitespace)
+        elif self.ch == 0:
+            token = Token(Tokens.EOF, "", comments, whitespace)
+        elif isExtendedAlpha(literal):
+            if literal == "b" and self._peekAtNextChar() == "6":
+                self.readChar()
+                self.readChar()
+                if chr(self.ch) == "4" and self._peekAtNextChar() == "'":
+                    self.readChar()
                     token = Token(
-                        Tokens.FLOAT if "." in numberOrFloat else Tokens.NUMBER,
-                        numberOrFloat,
+                        Tokens.BASE64,
+                        self._readBytesString(),
+                        comments,
+                        whitespace,
+                    )
+                else:
+                    # Looked like a b64 byte string, but that's just a regular
+                    # identifier in the end
+                    token = Token(
+                        Tokens.IDENT,
+                        self._readIdentifier("b6"),
                         comments,
                         whitespace,
                     )
                     tokenRead = True
+            elif literal == "h" and self._peekAtNextChar() == "'":
+                self.readChar()
+                token = Token(Tokens.HEX, self._readBytesString(), comments, whitespace)
+            else:
+                token = Token(
+                    Tokens.IDENT, self._readIdentifier(), comments, whitespace
+                )
+                tokenRead = True
+        elif literal.isdigit() or literal == "-":
+            # Numbers start with a digit or a minus
+            numberOrFloat = self._readNumberOrFloat()
+            token = Token(
+                Tokens.FLOAT if "." in numberOrFloat else Tokens.NUMBER,
+                numberOrFloat,
+                comments,
+                whitespace,
+            )
+            tokenRead = True
 
         if not tokenRead:
             self.readChar()
@@ -198,7 +190,7 @@ class Lexer:
 
         # see https://tools.ietf.org/html/draft-ietf-cbor-cddl-08#section-3.1
         if start == "" and not isExtendedAlpha(chr(self.ch)):
-            raise self._tokenError("identifier expected, found nothing")
+            raise self._tokenError("expected identifier, got nothing")
         while (
             isExtendedAlpha(chr(self.ch))
             or chr(self.ch).isdigit()
@@ -209,14 +201,14 @@ class Lexer:
         identifier = start + self.input[position : self.position]
         if identifier[-1] in "-.":
             raise self._tokenError(
-                'identifier cannot end with "-" or ".", found "{identifier}"'
+                f"identifier cannot end with `-` or `.`, got `{identifier}`"
             )
         return identifier
 
     def _readComment(self) -> str:
         position = self.position
 
-        while self.ch and chr(self.ch) != Tokens.NL:
+        while self.ch and chr(self.ch) != "\n":
             self.readChar()
 
         return self.input[position : self.position]
@@ -239,14 +231,18 @@ class Lexer:
                 if 0x20 <= self.ch <= 0x7E or 0x80 <= self.ch <= 0x10FFFD:
                     self.readChar()
                 else:
-                    raise self._tokenError("invalid escape character in text string")
+                    raise self._tokenError(
+                        f"invalid escape character in text string `{self.input[position + 1 : self.position]}`"
+                    )
             elif self.ch == 0x0A:
                 self.readChar()
             elif self.ch == 0x0D and ord(self._peekAtNextChar()) == 0x0A:
                 self.readChar()
                 self.readChar()
             else:
-                raise self._tokenError("invalid text string")
+                raise self._tokenError(
+                    f"invalid text string `{self.input[position + 1 : self.position]}`"
+                )
 
         return self.input[position + 1 : self.position]
 
@@ -267,14 +263,18 @@ class Lexer:
                 if 0x20 <= self.ch <= 0x7E or 0x80 <= self.ch <= 0x10FFFD:
                     self.readChar()
                 else:
-                    raise self._tokenError("invalid escape character in byte string")
+                    raise self._tokenError(
+                        f"invalid escape character in byte string `{self.input[position + 1 : self.position]}`"
+                    )
             elif self.ch == 0x0A:
                 self.readChar()
             elif self.ch == 0x0D and ord(self._peekAtNextChar()) == 0x0A:
                 self.readChar()
                 self.readChar()
             else:
-                raise self._tokenError("invalid byte string")
+                raise self._tokenError(
+                    f"invalid byte string `{self.input[position + 1 : self.position]}`"
+                )
 
         return self.input[position + 1 : self.position]
 
@@ -294,7 +294,9 @@ class Lexer:
                 # Hex number
                 self.readChar()
                 if chr(self.ch) not in "0123456789ABCDEF":
-                    raise self._tokenError("hex number detected but no hex digit found")
+                    raise self._tokenError(
+                        f"expected hex number to contain hex digits, got `{self.input[position : self.position]}`"
+                    )
                 while chr(self.ch) in "0123456789ABCDEF":
                     self.readChar()
                 if chr(self.ch) == ".":
@@ -308,7 +310,7 @@ class Lexer:
                         self.readChar()
                 if dotFound and chr(self.ch) != "p":
                     raise self._tokenError(
-                        "hex number with fraction detected but no exponent found"
+                        f"expected hex number with fraction to have an exponent, got `{self.input[position : self.position]}`"
                     )
                 if chr(self.ch) == "p":
                     # Number contains an exponent
@@ -317,7 +319,7 @@ class Lexer:
                         self.readChar()
                     if not chr(self.ch).isdigit():
                         raise self._tokenError(
-                            "hex number with exponent detected but no digit found for exponent"
+                            f"expected hex number with exponent to have a digit in exponent, got `{self.input[position : self.position]}`"
                         )
                     while chr(self.ch).isdigit():
                         self.readChar()
@@ -326,7 +328,7 @@ class Lexer:
                 self.readChar()
                 if chr(self.ch) not in "01":
                     raise self._tokenError(
-                        "binary number detected but no binary digit found"
+                        f"expected binary number to have binary digits, got `{self.input[position : self.position]}`"
                     )
                 while chr(self.ch) in "01":
                     self.readChar()
@@ -338,7 +340,7 @@ class Lexer:
                 self.readChar()
                 if not chr(self.ch).isdigit():
                     raise self._tokenError(
-                        "number with fraction detected but no digit found in fraction"
+                        f"expected number with fraction to have digits in fraction part, got `{self.input[position : self.position]}`"
                     )
                 while chr(self.ch).isdigit():
                     self.readChar()
@@ -349,7 +351,7 @@ class Lexer:
                         self.readChar()
                     if not chr(self.ch).isdigit():
                         raise self._tokenError(
-                            "number with exponent detected but no digit found in exponent"
+                            f"expected number with exponent to have digits in exponent, got `{self.input[position : self.position]}`"
                         )
                     while chr(self.ch).isdigit():
                         self.readChar()
@@ -367,7 +369,7 @@ class Lexer:
                 self.readChar()
                 if not chr(self.ch).isdigit():
                     raise self._tokenError(
-                        "number with fraction detected but no digit found in fraction"
+                        f"expected number with fraction to have digits in fraction part, got `{self.input[position : self.position]}`"
                     )
                 while chr(self.ch).isdigit():
                     self.readChar()
@@ -378,7 +380,7 @@ class Lexer:
                     self.readChar()
                 if not chr(self.ch).isdigit():
                     raise self._tokenError(
-                        "number with exponent detected but no digit found in exponent"
+                        f"expected number with exponent to have digits in exponent, got `{self.input[position : self.position]}`"
                     )
                 while chr(self.ch).isdigit():
                     self.readChar()
@@ -409,7 +411,4 @@ class Lexer:
 
     def _tokenError(self, message: str) -> ParserError:
         location = self.getLocation()
-        locInfo = self.getLocationInfo()
-        return ParserError(
-            f"CDDL token error - line {location.line + 1}, col {location.position}: {message}\n\n{locInfo}"
-        )
+        return ParserError(f"CDDL token error - line {location.line + 1}: {message}")
