@@ -1,4 +1,4 @@
-from typing import get_args
+from typing import get_args, Union, Optional
 from math import inf
 
 from .errors import ParserError
@@ -90,7 +90,7 @@ class Parser:
             node = Rule(typename, assign, ruleType)
         else:
             raise self._parserError(
-                f'assignment expected, received "{assign.serialize()}"'
+                f"expected assignment (`=`, `/=`, `//=`) after `{typename.serialize().strip()}`, got `{assign.serialize().strip()}`"
             )
         return node
 
@@ -117,7 +117,7 @@ class Parser:
             node = GroupEntry(occurrence, None, looseType)
         return node
 
-    def _parseType(self, loose: bool = False) -> Type | Memberkey:
+    def _parseType(self, loose: bool = False) -> Union[Type, Memberkey]:
         """
         type = type1 *(S "/" S type1)
 
@@ -137,7 +137,7 @@ class Parser:
             caretTokens.append(self._nextToken())
             if self.curToken.type != Tokens.ARROWMAP:
                 raise self._parserError(
-                    f'expected arrow map, received "{self.curToken.serialize()}{self.peekToken.serialize()}"'
+                    f"expected arrow map (`=>`), got `{(self.curToken.serialize() + self.peekToken.serialize()).strip()}`"
                 )
             caretTokens.append(self._nextToken())
             key = Memberkey(type1, hasCut=True, hasColon=False, tokens=caretTokens)
@@ -178,17 +178,17 @@ class Parser:
             maxType = self._parseType2()
             if not isinstance(type2, (Value, Typename)):
                 raise self._parserError(
-                    f"range detected but min is neither a value nor a typename. Got: {type2.serialize()}"
+                    f"expected range min to be a value or a typename, got `{type2.serialize().strip()}`"
                 )
             if not isinstance(maxType, (Value, Typename)):
                 raise self._parserError(
-                    f"range detected but max is neither a value nor a typename. Got: {maxType.serialize()}"
+                    f"expected range max to be a value or a typename, got `{maxType.serialize().strip()}`"
                 )
             node = Range(type2, maxType, rangeop)
         elif self.curToken.type == Tokens.CTLOP:
             if self.curToken.literal not in get_args(OperatorName):
                 raise self._parserError(
-                    f'unknown control operator "{self.curToken.literal}"'
+                    f"found unknown control operator `{self.curToken.literal.strip()}`"
                 )
             operator = self._nextToken()
             controlType = self._parseType2()
@@ -217,128 +217,127 @@ class Parser:
               / "(" S group S ")"
         """
         node: Type2
-        match self.curToken.type:
-            case Tokens.LPAREN:
-                openToken = self._nextToken()
-                if loose:
-                    node = self._parseGroup(isMap=False)
-                else:
-                    innerType = self._parseType()
-                    assert isinstance(innerType, Type)
-                    node = innerType
-                node.openToken = openToken
-                if self.curToken.type != Tokens.RPAREN:
-                    raise self._parserError(
-                        f'expected right parenthesis, received "{self.curToken.serialize()}"'
-                    )
-                node.closeToken = self._nextToken()
+        if self.curToken.type == Tokens.LPAREN:
+            openToken = self._nextToken()
+            if loose:
+                node = self._parseGroup(isMap=False)
+            else:
+                innerType = self._parseType()
+                assert isinstance(innerType, Type)
+                node = innerType
+            node.openToken = openToken
+            if self.curToken.type != Tokens.RPAREN:
+                raise self._parserError(
+                    f"expected right parenthesis, got `{self.curToken.serialize().strip()}`"
+                )
+            node.closeToken = self._nextToken()
 
-            case Tokens.LBRACE:
-                openToken = self._nextToken()
-                node = self._parseGroup(isMap=True)
-                node.openToken = openToken
-                if self.curToken.type != Tokens.RBRACE:
-                    raise self._parserError(
-                        f'expected right brace, received "{self.curToken.serialize()}"'
-                    )
-                node.closeToken = self._nextToken()
+        elif self.curToken.type == Tokens.LBRACE:
+            openToken = self._nextToken()
+            node = self._parseGroup(isMap=True)
+            node.openToken = openToken
+            if self.curToken.type != Tokens.RBRACE:
+                raise self._parserError(
+                    f"expected right brace, got `{self.curToken.serialize()}`"
+                )
+            node.closeToken = self._nextToken()
 
-            case Tokens.LBRACK:
+        elif self.curToken.type == Tokens.LBRACK:
+            openToken = self._nextToken()
+            group = self._parseGroup(isMap=False)
+            node = Array(group.groupChoices)
+            node.openToken = openToken
+            if self.curToken.type != Tokens.RBRACK:
+                raise self._parserError(
+                    f"expected right bracket, got `{self.curToken.serialize().strip()}`"
+                )
+            node.closeToken = self._nextToken()
+
+        elif self.curToken.type == Tokens.TILDE:
+            unwrapped = self._nextToken()
+            node = self._parseTypename(definition=False, unwrapped=unwrapped)
+
+        elif self.curToken.type == Tokens.AMPERSAND:
+            refToken = self._nextToken()
+            if self.curToken.type == Tokens.LPAREN:
                 openToken = self._nextToken()
                 group = self._parseGroup(isMap=False)
-                node = Array(group.groupChoices)
-                node.openToken = openToken
-                if self.curToken.type != Tokens.RBRACK:
+                group.openToken = openToken
+                if self.curToken.type != Tokens.RPAREN:
                     raise self._parserError(
-                        f'expected right bracket, received "{self.curToken.serialize()}"'
+                        f"expected right parenthesis, got `{self.curToken.serialize().strip()}`"
                     )
-                node.closeToken = self._nextToken()
+                group.closeToken = self._nextToken()
+                node = ChoiceFrom(group)
+            else:
+                typename = self._parseTypename(definition=False, unwrapped=None)
+                node = ChoiceFrom(typename)
+            node.setComments(refToken)
 
-            case Tokens.TILDE:
-                unwrapped = self._nextToken()
-                node = self._parseTypename(definition=False, unwrapped=unwrapped)
-
-            case Tokens.AMPERSAND:
-                refToken = self._nextToken()
-                if self.curToken.type == Tokens.LPAREN:
-                    openToken = self._nextToken()
-                    group = self._parseGroup(isMap=False)
-                    group.openToken = openToken
-                    if self.curToken.type != Tokens.RPAREN:
-                        raise self._parserError(
-                            f'expected right parenthesis, received "{self.curToken.serialize()}"'
-                        )
-                    group.closeToken = self._nextToken()
-                    node = ChoiceFrom(group)
-                else:
-                    typename = self._parseTypename(definition=False, unwrapped=None)
-                    node = ChoiceFrom(typename)
-                node.setComments(refToken)
-
-            case Tokens.HASH:
-                hashToken = self._nextToken()
+        elif self.curToken.type == Tokens.HASH:
+            hashToken = self._nextToken()
+            if (
+                self.curToken.type in {Tokens.NUMBER, Tokens.FLOAT}
+                and not self.curToken.startWithSpaces()
+            ):
+                number = self._nextToken()
+                if len(number.literal) > 1 and (
+                    number.literal[1] != "." or "e" in number.literal
+                ):
+                    raise self._parserError(
+                        f'expected data item after "#" to match `DIGIT ["." uint]`, got `{self.curToken.serialize().strip()}`'
+                    )
                 if (
-                    self.curToken.type in {Tokens.NUMBER, Tokens.FLOAT}
+                    number.literal[0] == "6"
+                    and self.curToken.type == Tokens.LPAREN
                     and not self.curToken.startWithSpaces()
                 ):
-                    number = self._nextToken()
-                    if len(number.literal) > 1 and (
-                        number.literal[1] != "." or "e" in number.literal
-                    ):
-                        raise self._parserError(
-                            f'data item after "#" must match DIGIT ["." uint], got "{self.curToken.serialize()}"'
-                        )
-                    if (
-                        number.literal[0] == "6"
-                        and self.curToken.type == Tokens.LPAREN
-                        and not self.curToken.startWithSpaces()
-                    ):
-                        type2 = self._parseType2()
-                        assert isinstance(type2, Type)
-                        node = Tag(number, type2)
-                    else:
-                        node = Tag(number)
+                    type2 = self._parseType2()
+                    assert isinstance(type2, Type)
+                    node = Tag(number, type2)
                 else:
-                    node = Tag()
-                node.setComments(hashToken)
+                    node = Tag(number)
+            else:
+                node = Tag()
+            node.setComments(hashToken)
 
-            case Tokens.IDENT:
-                node = self._parseTypename(definition=False, unwrapped=None)
+        elif self.curToken.type == Tokens.IDENT:
+            node = self._parseTypename(definition=False, unwrapped=None)
 
-            case Tokens.STRING:
-                value = self._nextToken()
-                node = Value(value.literal, "text")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.STRING:
+            value = self._nextToken()
+            node = Value(value.literal, "text")
+            node.setComments(value)
 
-            case Tokens.BYTES:
-                value = self._nextToken()
-                node = Value(value.literal, "bytes")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.BYTES:
+            value = self._nextToken()
+            node = Value(value.literal, "bytes")
+            node.setComments(value)
 
-            case Tokens.HEX:
-                value = self._nextToken()
-                node = Value(value.literal, "hex")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.HEX:
+            value = self._nextToken()
+            node = Value(value.literal, "hex")
+            node.setComments(value)
 
-            case Tokens.BASE64:
-                value = self._nextToken()
-                node = Value(value.literal, "base64")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.BASE64:
+            value = self._nextToken()
+            node = Value(value.literal, "base64")
+            node.setComments(value)
 
-            case Tokens.NUMBER:
-                value = self._nextToken()
-                node = Value(value.literal, "number")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.NUMBER:
+            value = self._nextToken()
+            node = Value(value.literal, "number")
+            node.setComments(value)
 
-            case Tokens.FLOAT:
-                value = self._nextToken()
-                node = Value(value.literal, "number")
-                node.setComments(value)
+        elif self.curToken.type == Tokens.FLOAT:
+            value = self._nextToken()
+            node = Value(value.literal, "number")
+            node.setComments(value)
 
-            case _:
-                raise self._parserError(
-                    f'invalid type2 production, received "{self.curToken.serialize()}"'
-                )
+        else:
+            raise self._parserError(
+                f"invalid type2 production, got `{self.curToken.serialize().strip()}`"
+            )
 
         return node
 
@@ -369,16 +368,16 @@ class Parser:
                 break
             groupChoice.separator = self._nextToken()
 
-        node: Map | Group
+        node: Union[Map, Group]
         if isMap:
             node = Map(groupChoices)
         else:
             node = Group(groupChoices)
         return node
 
-    def _parseOccurrence(self) -> Occurrence | None:
+    def _parseOccurrence(self) -> Optional[Occurrence]:
         tokens: list[Token] = []
-        occurrence: Occurrence | None = None
+        occurrence: Optional[Occurrence] = None
 
         # check for non-numbered occurrence indicator, e.g.
         # ```
@@ -438,14 +437,14 @@ class Parser:
         return occurrence
 
     def _parseTypename(
-        self, definition: bool = False, unwrapped: Token | None = None
+        self, definition: bool = False, unwrapped: Optional[Token] = None
     ) -> Typename:
         if self.curToken.type != Tokens.IDENT:
             raise self._parserError(
-                f'group identifier expected, received "{self.curToken.serialize()}"'
+                f"expected group identifier, got `{self.curToken.serialize().strip()}`"
             )
         ident = self._nextToken()
-        parameters: GenericParameters | GenericArguments | None
+        parameters: Union[GenericParameters, GenericArguments, None]
         if definition:
             parameters = self._parseGenericParameters()
         else:
@@ -454,7 +453,7 @@ class Parser:
         typename.setComments(ident)
         return typename
 
-    def _parseGenericParameters(self) -> GenericParameters | None:
+    def _parseGenericParameters(self) -> Optional[GenericParameters]:
         """
         genericparm = "<" S id S *("," S id S ) ">"
         """
@@ -474,12 +473,12 @@ class Parser:
         node.openToken = openToken
         if self.curToken.type != Tokens.GT:
             raise self._parserError(
-                f'">" character expected to end generic production, received "{self.curToken.serialize()}"'
+                f"expected `>` character to end generic production, got `{self.curToken.serialize().strip()}`"
             )
         node.closeToken = self._nextToken()
         return node
 
-    def _parseGenericArguments(self) -> GenericArguments | None:
+    def _parseGenericArguments(self) -> Optional[GenericArguments]:
         """
         genericarg = "<" S type1 S *("," S type1 S ) ">"
 
@@ -502,7 +501,7 @@ class Parser:
         node.openToken = openToken
         if self.curToken.type != Tokens.GT:
             raise self._parserError(
-                f'">" character expected to end generic production, received "{self.curToken.serialize()}"'
+                f"expected `>` character to end generic production, got `{self.curToken.serialize().strip()}`"
             )
         node.closeToken = self._nextToken()
         return node
@@ -589,7 +588,7 @@ class Parser:
         # Recursive check in the tree to look for typenames that are used as
         # group keys (real typenames, not barewords). Such keys are types, see:
         # https://datatracker.ietf.org/doc/html/rfc8610#section-2.1.2
-        def lookForKeys(node: CDDLNode):
+        def lookForKeys(node: CDDLNode) -> None:
             if (
                 isinstance(node, GroupEntry)
                 and node.key is not None
@@ -639,8 +638,8 @@ class Parser:
                         checkUnderlyingType(type1) for type1 in rule.type.type.types
                     }
                     if "type" in defTypes and "group" in defTypes:
-                        raise self._parserError(
-                            f'rule "{rule.name.name}" targets a mix of type and group rules'
+                        raise ParserError(
+                            f"CDDL semantic error - rule `{rule.name.name}` targets a mix of type and group rules"
                         )
                     if "type" in defTypes:
                         updateFound = rule.name.name not in typenames
@@ -653,8 +652,8 @@ class Parser:
         overlap = list(set(typenames) & set(groupnames))
         if len(overlap) > 0:
             overlapStr = ", ".join(overlap)
-            raise self._parserError(
-                f"mix of type and group definitions for {overlapStr}"
+            raise ParserError(
+                f"CDDL semantic error - mix of type and group definitions for {overlapStr}"
             )
 
         # Convert GroupEntry to Type for type definitions
@@ -664,8 +663,8 @@ class Parser:
             assert isinstance(rule.type, GroupEntry)
             if rule.name.name in typenames:
                 if not rule.type.isConvertibleToType():
-                    raise self._parserError(
-                        f'rule "{rule.name.name}" is a type definition but uses a group entry'
+                    raise ParserError(
+                        f"CDDL semantic error - rule `{rule.name.name}` is a type definition but uses a group entry"
                     )
                 rule.type = rule.type.type
 
@@ -677,7 +676,4 @@ class Parser:
 
     def _parserError(self, message: str) -> ParserError:
         location = self.lexer.getLocation()
-        locInfo = self.lexer.getLocationInfo()
-        return ParserError(
-            f"CDDL syntax error - line {location.line + 1}, col {location.position}: {message}\n\n{locInfo}"
-        )
+        return ParserError(f"CDDL syntax error - line {location.line + 1}: {message}")
